from pylab import *

rc('text', usetex = True)
rc('font', family = 'serif')
rc('font', size = 12)
rcParams['text.latex.preamble']=[r'\usepackage{amsmath}']



def continualize(x):
    n = len(x)
    for i in range(n - 1):
        if x[i + 1] - x[i] < - 6: # somewhat arbitrary
            for j in range(i + 1, n):
                x[j] += 2 * pi
        elif x[i + 1] - x[i] > 6: # somewhat arbitrary
            for j in range(i + 1, n):
                x[j] -= 2 * pi
    return x



R = 5
deg = linspace(0, 360, 361)
theta = radians(deg)
z1 = R * exp(1j * theta)


nf = 16

def f(z):
    return - 50 * (z - 1) * (z + 7) * (z - 1j) * (z + 1j) / (z**4 * (z - 8) * (z + 2j) * (z - 2j))

def fname():
    return '- 50 \, \dfrac{(z - 1) \, (z + 7) \, (z - 1j) \, (z + 1j)}{z^4 \, (z - 8) \, (z + 2j) \, (z - 2j)}'
    
    
    
z2 = array(list(map(f, z1)))

r = abs(z2)
rho = log(r)
phi = degrees(continualize(angle(z2)))

zmp = int(round((phi[-1] - phi[0]) / 360))

print()
print('Z - P = {:d}'.format(zmp))
print()



close('all')


figure(1, figsize = (9, 9))

subplot(2, 1, 1)
plot(deg, rho)
ylabel(r'$\ln \left| {f} \left( z \left( \theta \right) \right) \right|$')
xlim(0, 360)
xticks(linspace(0, 360, int(360 / 30) + 1))
grid(True)
#title('$f(z) = ' + fname() + '$')

subplot(2, 1, 2)
plot(deg, phi)
ylabel(r'$\operatorname{arg} \left(f \left( z \left( \theta \right) \right) \right)  \; [^\circ]$')
yticks((0, -360, -720, -1080))
xlabel(r'$\theta \; [^\circ]$')
xlim(0, 360)
xticks(linspace(0, 360, int(360 / 30) + 1))
grid(True)

savefig('figs/primer1ap.pdf', bbox_inches = 'tight')


figure(2, figsize = (6, 6))

#title('$f(z) = ' + fname() + '$')

plot(r * cos(radians(phi)), r * sin(radians(phi)))
plot(0, 0, 'r+')
xlim(-1.25, 1.25)
ylim(-1.25, 1.25)
xlabel(r'$\Re \left( f \left( z \left( \theta \right) \right) \right)$')
ylabel(r'$\Im \left( f \left( z \left( \theta \right) \right) \right)$')
grid(True)

savefig('figs/primer1polar.pdf', bbox_inches = 'tight')


figure(3, figsize = (6, 6))

#title('$f(z) = ' + fname() + '$')

plot(z1.real, z1.imag)
plot(0, 0, 'r+')
xlim(-6, 6)
ylim(-6, 6)
xlabel(r'$\Re \left( z \left( \theta \right) \right)$')
ylabel(r'$\Im \left( z \left( \theta \right) \right)$')
grid(True)

savefig('figs/primer1contour-polar.pdf', bbox_inches = 'tight')


figure(4, figsize = (9,9))

subplot(2, 1, 1)
plot(deg, abs(z1))
ylabel(r'$\left| z \left( \theta \right) \right|$')
ylim(0, 6)
xlim(0, 360)
xticks(linspace(0, 360, int(360 / 30) + 1))
grid(True)
#title('$f(z) = ' + fname() + '$')

subplot(2, 1, 2)
plot(deg, degrees(theta))
ylabel(r'$\operatorname{arg} \left( z \left( \theta \right) \right) \; [^\circ]$')
ylim(0, 360)
yticks((0, 90, 180, 270, 360))
xlabel(r'$\theta \; [^\circ]$')
xlim(0, 360)
xticks(linspace(0, 360, int(360 / 30) + 1))
grid(True)

savefig('figs/primer1contour.pdf', bbox_inches = 'tight')

